@@ -24,6 +24,7 @@
 #include <string.h>
 
 /* X11 headers */
+#include <X11/Xlib.h>
 #ifdef HAVE_XFREE
 #include <X11/XF86keysym.h>
 #endif
@@ -113,7 +114,7 @@ static char
 
 	for (i = 0; languages[i].code != NULL; i++)
 	{
-		if (strcmp (code, languages[i].code) == 0)
+		if (strstr (code, languages[i].code) != NULL)
 			return g_strdup (languages[i].language);
 	}
 
@@ -134,7 +135,7 @@ totem_g_list_deep_free (GList *list)
 static char*
 totem_create_full_path (const char *path)
 {
-	char *retval, *curdir, *curdir_withslash;
+	char *retval, *curdir, *curdir_withslash, *escaped;
 
 	g_return_val_if_fail (path != NULL, NULL);
 
@@ -142,12 +143,22 @@ totem_create_full_path (const char *path)
 		return g_strdup (path);
 
 	if (path[0] == '/')
-		return g_strdup (path);
+	{
+		escaped = gnome_vfs_escape_path_string (path);
+		retval = g_strdup_printf ("file://%s", escaped);
+		g_free (escaped);
+		return retval;
+	}
 
 	curdir = g_get_current_dir ();
-	curdir_withslash = g_strdup_printf ("%s%s", curdir, G_DIR_SEPARATOR_S);
+	curdir_withslash = g_strdup_printf ("file:///%s%s", curdir, G_DIR_SEPARATOR_S);
 	g_free (curdir);
-	retval = gnome_vfs_uri_make_full_from_relative (curdir_withslash, path);        g_free (curdir_withslash);
+
+	escaped = gnome_vfs_escape_path_string (path);
+	retval = gnome_vfs_uri_make_full_from_relative (curdir_withslash,
+			escaped);
+	g_free (curdir_withslash);
+	g_free (escaped);
 
 	return retval;
 }
@@ -174,8 +185,6 @@ totem_action_error (char *msg, Totem *totem)
 			(gtk_widget_destroy), error_dialog);
 	g_signal_connect (G_OBJECT (error_dialog), "response", G_CALLBACK
 			(gtk_widget_destroy), error_dialog);
-	g_object_add_weak_pointer (G_OBJECT (error_dialog),
-			(void**)&(error_dialog));
 	gtk_window_set_modal (GTK_WINDOW (error_dialog), TRUE);
 
 	gtk_widget_show (error_dialog);
@@ -335,8 +344,6 @@ totem_action_play (Totem *totem, int offset)
 			totem_action_stop (totem);
 		g_free (msg);
 	}
-
-	update_dvd_menu_sub_lang (totem);
 }
 
 void
@@ -485,7 +492,7 @@ totem_get_nice_name_for_stream (Totem *totem)
 	return retval;
 }
 
-static gboolean
+static void
 totem_action_restore_pl (Totem *totem)
 {
 	char *path, *mrl;
@@ -496,25 +503,44 @@ totem_action_restore_pl (Totem *totem)
 	if (g_file_test (path, G_FILE_TEST_EXISTS) == FALSE)
 	{
 		g_free (path);
-		return FALSE;
+		return;
 	}
 
+	g_signal_handlers_disconnect_by_func
+		(G_OBJECT (totem->playlist),
+		 playlist_changed_cb, totem);
+
 	if (gtk_playlist_add_mrl (totem->playlist, path, NULL) == FALSE)
 	{
+		g_signal_connect (G_OBJECT (totem->playlist),
+				"changed", G_CALLBACK (playlist_changed_cb),
+				totem);
+
 		g_free (path);
-		return FALSE;
+		totem_action_set_mrl (totem, NULL);
+		return;
 	}
 
-	g_free (path);
-	mrl = gtk_playlist_get_current_mrl (totem->playlist);
-	if (mrl == NULL)
-		return FALSE;
+	g_signal_connect (G_OBJECT (totem->playlist),
+			"changed", G_CALLBACK (playlist_changed_cb),
+			totem);
 
-	totem_action_set_mrl (totem, mrl);
+	g_free (path);
 	play_pause_set_label (totem, STATE_PAUSED);
+
+	mrl = gtk_playlist_get_current_mrl (totem->playlist);
+	if (totem->mrl == NULL
+			|| (totem->mrl != NULL && mrl != NULL
+				&& strcmp (totem->mrl, mrl) != 0))
+	{
+		totem_action_set_mrl (totem, mrl);
+	} else if (totem->mrl != NULL) {
+		gtk_playlist_set_playing (totem->playlist, TRUE);
+	}
+
 	g_free (mrl);
 
-	return TRUE;
+	return;
 }
 
 static void
@@ -531,6 +557,7 @@ update_mrl_label (Totem *totem, const char *name)
 		/* Get the length of the stream */
 		time = bacon_video_widget_get_stream_length (totem->bvw);
 
+		//FIXME this doesn't work if the metadata arrives later
 		totem_statusbar_set_time_and_length (TOTEM_STATUSBAR
 				(totem->statusbar), 0, time / 1000);
 
@@ -559,7 +586,7 @@ update_mrl_label (Totem *totem, const char *name)
 		g_free (text);
 	} else {
 		totem_statusbar_set_time_and_length (TOTEM_STATUSBAR
-				(totem->statusbar), 0, 0);
+				(totem->statusbar), 0, -1);
 
 		widget = glade_xml_get_widget (totem->xml, "spinbutton1");
 		gtk_spin_button_set_range (GTK_SPIN_BUTTON (widget), 0, 0);
@@ -596,6 +623,11 @@ totem_action_set_mrl (Totem *totem, const char *mrl)
 		bacon_video_widget_close (totem->bvw);
 	}
 
+	/* Reset the properties and wait for the signal*/
+	bacon_video_widget_properties_update
+		(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
+		 totem->bvw, TRUE);
+
 	if (mrl == NULL)
 	{
 		retval = FALSE;
@@ -610,8 +642,6 @@ totem_action_set_mrl (Totem *totem, const char *mrl)
 		widget = glade_xml_get_widget (totem->xml, "play2");
 		gtk_widget_set_sensitive (widget, FALSE);
 
-		update_mrl_label (totem, NULL);
-
 		/* Seek bar and seek buttons */
 		update_seekable (totem, TRUE);
 
@@ -647,10 +677,7 @@ totem_action_set_mrl (Totem *totem, const char *mrl)
 		bacon_video_widget_set_logo (totem->bvw, totem->mrl);
 		bacon_video_widget_set_logo_mode (totem->bvw, TRUE);
 
-		/* Reset the properties */
-		bacon_video_widget_properties_update
-			(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
-			 totem->bvw, NULL, TRUE);
+		update_mrl_label (totem, NULL);
 	} else {
 		char *name;
 		gboolean caps, custom;
@@ -701,11 +728,6 @@ totem_action_set_mrl (Totem *totem, const char *mrl)
 		/* Set the playlist */
 		gtk_playlist_set_playing (totem->playlist, retval);
 
-		/* Update the properties */
-		bacon_video_widget_properties_update
-			(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
-			 totem->bvw, name, !retval);
-
 		g_free (name);
 
 		if (retval == FALSE)
@@ -907,7 +929,7 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 				 * changed playlist ... */
 				g_signal_handlers_disconnect_by_func
 					(G_OBJECT (totem->playlist),
-					 playlist_changed_cb, (gpointer) totem);
+					 playlist_changed_cb, totem);
 				gtk_playlist_clear (totem->playlist);
 				cleared = TRUE;
 			}
@@ -926,7 +948,7 @@ totem_action_drop_files (Totem *totem, GtkSelectionData *data,
 
 		g_signal_connect (G_OBJECT (totem->playlist),
 				"changed", G_CALLBACK (playlist_changed_cb),
-				(gpointer) totem);
+				totem);
 		mrl = gtk_playlist_get_current_mrl (totem->playlist);
 		totem_action_set_mrl_and_play (totem, mrl);
 		g_free (mrl);
@@ -1059,11 +1081,102 @@ on_title_change_event (GtkWidget *win, const char *string, Totem *totem)
 {
 	update_mrl_label (totem, string);
 	gtk_playlist_set_title (GTK_PLAYLIST (totem->playlist), string);
+}
 
-	/* The DVD plugin changes the title when loading new bits */
+static void
+on_channels_change_event (BaconVideoWidget *bvw, Totem *totem)
+{
 	update_dvd_menu_sub_lang (totem);
 }
 
+static void
+on_got_metadata_event (BaconVideoWidget *bvw, Totem *totem)
+{
+	bacon_video_widget_properties_update
+		(BACON_VIDEO_WIDGET_PROPERTIES (totem->properties),
+		 totem->bvw, FALSE);
+}
+
+static int
+on_buffering_cancel_event (GtkWidget *dialog, int response, Totem *totem)
+{
+	bacon_video_widget_close (totem->bvw);
+	play_pause_set_label (totem, STATE_PAUSED);
+
+	gtk_widget_destroy (dialog);
+	totem->buffer_dialog = NULL;
+	totem->buffer_label = NULL;
+
+	g_free (totem->mrl);
+	totem->mrl = NULL;
+
+	return TRUE;
+}
+
+static void
+on_buffering_event (BaconVideoWidget *bvw, int percentage, Totem *totem)
+{
+	char *msg;
+
+	if (percentage == 0)
+	{
+		GtkWidget *image, *hbox;
+
+		totem->buffer_dialog = gtk_dialog_new_with_buttons (_("Buffering"),
+				GTK_WINDOW (totem->win),
+				GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_STOCK_CANCEL,
+				GTK_RESPONSE_REJECT,
+				NULL);
+		gtk_dialog_set_has_separator (GTK_DIALOG(totem->buffer_dialog),
+				FALSE);
+		gtk_window_set_modal (GTK_WINDOW (totem->buffer_dialog), TRUE);
+		gtk_container_set_border_width
+			(GTK_CONTAINER (GTK_DIALOG(totem->buffer_dialog)->vbox),
+			 8);
+		hbox = gtk_hbox_new (FALSE, 12);
+		gtk_container_add
+			(GTK_CONTAINER (GTK_DIALOG(totem->buffer_dialog)->vbox),
+			 hbox);
+
+		image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO,
+				GTK_ICON_SIZE_DIALOG);
+		gtk_container_add (GTK_CONTAINER (hbox), image);
+
+		msg = g_strdup_printf (_("Buffering: %d%%"), percentage);
+		totem->buffer_label = gtk_label_new (msg);
+		g_free (msg);
+		gtk_container_add (GTK_CONTAINER(hbox), totem->buffer_label);
+
+		g_signal_connect (G_OBJECT (totem->buffer_dialog),
+				"response",
+				G_CALLBACK (on_buffering_cancel_event),
+				totem);
+		g_signal_connect (G_OBJECT (totem->buffer_dialog),
+				"delete-event",
+				G_CALLBACK (on_buffering_cancel_event),
+				totem);
+
+		gtk_widget_show_all (totem->buffer_dialog);
+		long_action ();
+
+		//FIXME connect cancel
+	} else if (percentage == 100) {
+		if (totem->buffer_dialog != NULL)
+			gtk_widget_destroy (totem->buffer_dialog);
+		totem->buffer_dialog = NULL;
+		totem->buffer_label = NULL;
+	} else {
+		if (totem->buffer_label == NULL)
+			return;
+
+		msg = g_strdup_printf (_("Buffering: %d%%"), percentage);
+		gtk_label_set_text (GTK_LABEL (totem->buffer_label), msg);
+		g_free (msg);
+		long_action ();
+	}
+}
+
 static void
 update_seekable (Totem *totem, gboolean force_false)
 {
@@ -1097,8 +1210,17 @@ static void
 update_current_time (BaconVideoWidget *bvw, int current_time, int stream_length,
 		int current_position, Totem *totem)
 {
-	totem_statusbar_set_time_and_length (TOTEM_STATUSBAR (totem->statusbar),
+	if (bacon_video_widget_get_logo_mode (totem->bvw) == TRUE
+			|| (current_time == 0 && stream_length == 0))
+	{
+		totem_statusbar_set_time_and_length
+			(TOTEM_STATUSBAR (totem->statusbar),
+			 current_time / 1000, -1);
+	} else {
+		totem_statusbar_set_time_and_length
+			(TOTEM_STATUSBAR (totem->statusbar),
 			current_time / 1000, stream_length / 1000);
+	}
 
 	if (totem->seek_lock == FALSE)
 	{
@@ -1228,27 +1350,19 @@ totem_action_open_files (Totem *totem, char **list, gboolean ignore_first)
 
 	for ( ; list[i] != NULL; i++)
 	{
-		char *filename, *subtitle;
+		char *filename;
 
 		/* Get the subtitle part out for our tests */
 		filename = totem_create_full_path (list[i]);
-		subtitle = strrchr (filename, '#');
-		if (subtitle != NULL)
-		{
-			*subtitle = 0;
-			subtitle++;
-		}
 
 		if (g_file_test (filename, G_FILE_TEST_IS_REGULAR)
+				|| strstr (filename, "#") != NULL
 				|| strstr (filename, "://") != NULL
 				|| strncmp (filename, "dvd:", 4) == 0
 				|| strncmp (filename, "vcd:", 4) == 0
 				|| strncmp (filename, "cdda:", 5) == 0
 				|| strncmp (filename, "cd:", 3) == 0)
 		{
-			g_free (filename);
-			filename = totem_create_full_path (list[i]);
-
 			if (cleared == FALSE)
 			{
 				/* The function that calls us knows better
@@ -1256,7 +1370,7 @@ totem_action_open_files (Totem *totem, char **list, gboolean ignore_first)
 				 * changed playlist ... */
 				g_signal_handlers_disconnect_by_func
 					(G_OBJECT (totem->playlist),
-					 playlist_changed_cb, (gpointer) totem);
+					 playlist_changed_cb, totem);
 				gtk_playlist_clear (totem->playlist);
 				cleared = TRUE;
 			}
@@ -1273,29 +1387,19 @@ totem_action_open_files (Totem *totem, char **list, gboolean ignore_first)
 			} else if (gtk_playlist_add_mrl (totem->playlist,
 						filename, NULL) == TRUE)
 			{
-                                char *uri;
                                 EggRecentItem *item;
 
-				if (filename[0] != '/')
+				if (strstr (filename, "file:///") == NULL)
 					continue;
 
-				uri = gnome_vfs_get_uri_from_local_path
-					(filename);
-
-				if (uri == NULL) {
-					/* ok, if this fails, then it was
-					 * something like dvd:/// and we don't
-					 * want to add it
-					 */
-					continue;
-				}
-
-				item = egg_recent_item_new_from_uri (uri);
+				//FIXME egg recent seems to unescape our
+				//pure and clean uri
+				//playing something with a # in the name
+				//won't get saved properly
+				item = egg_recent_item_new_from_uri (filename);
 				egg_recent_item_add_group (item, "Totem");
 				egg_recent_model_add_full (totem->recent_model,
 						item);
-
-				g_free (uri);
 			}
 		}
 
@@ -1307,7 +1411,7 @@ totem_action_open_files (Totem *totem, char **list, gboolean ignore_first)
 	{
 		g_signal_connect (G_OBJECT (totem->playlist),
 				"changed", G_CALLBACK (playlist_changed_cb),
-				(gpointer) totem);
+				totem);
 	}
 
 	return cleared;
@@ -1364,7 +1468,7 @@ on_open_location1_activate (GtkButton *button, Totem *totem)
 	char *filename, *mrl;
 	GtkWidget *dialog, *entry;
 	int response;
-	const char *filenames[1];
+	const char *filenames[2];
 
 	filename = gnome_program_locate_file (NULL,
 			GNOME_FILE_DOMAIN_APP_DATADIR,
@@ -1402,6 +1506,7 @@ on_open_location1_activate (GtkButton *button, Totem *totem)
 		if (uri != NULL && strcmp (uri, "") != 0)
 		{
 			filenames[0] = uri;
+			filenames[1] = NULL;
 			totem_action_open_files (totem,
 					(char **) filenames, FALSE);
 
@@ -2044,7 +2149,7 @@ playlist_repeat_toggle_cb (GtkPlaylist *playlist, gboolean repeat, Totem *totem)
 	item = glade_xml_get_widget (totem->xml, "repeat_mode1");
 
 	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
-			on_repeat_mode1_toggled, (gpointer) totem);
+			on_repeat_mode1_toggled, totem);
 
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), repeat);
 
@@ -2268,13 +2373,13 @@ totem_action_handle_key (Totem *totem, GdkEventKey *event)
 	case GDK_n:
 		totem_action_next (totem);
 		if (totem->action == 5)
-			totem_action_set_mrl_and_play (totem, "v4l://");
+			totem_action_set_mrl_and_play (totem, "v4l:/");
 		totem->action = 0;
 		break;
 	case GDK_O:
 	case GDK_o:
 		totem_action_fullscreen (totem, FALSE);
-		on_open1_activate (NULL, (gpointer) totem);
+		on_open1_activate (NULL, totem);
 		if (totem->action == 4)
 			totem->action++;
 		else
@@ -2413,7 +2518,7 @@ update_media_menu_items (Totem *totem)
 
 	playing = totem_playing_dvd (totem);
 
-        item = glade_xml_get_widget (totem->xml, "dvd_root_menu");
+	item = glade_xml_get_widget (totem->xml, "dvd_root_menu");
 	gtk_widget_set_sensitive (item, playing);
         item = glade_xml_get_widget (totem->xml, "dvd_title_menu");
 	gtk_widget_set_sensitive (item, playing);
@@ -2540,7 +2645,6 @@ create_submenu (Totem *totem, GList *list, int current, gboolean is_lang)
 
 	for (l = list; l != NULL; l = l->next)
 	{
-		GtkWidget *menuitem;
 		char *lang;
 
 		if (menu == NULL)
@@ -2581,13 +2685,11 @@ update_dvd_menu_sub_lang (Totem *totem)
 
 		list = bacon_video_widget_get_languages (totem->bvw);
 		current = bacon_video_widget_get_language (totem->bvw);
-		g_message ("current_lang: %d", current);
 		lang_menu = create_submenu (totem, list, current, TRUE);
 		totem_g_list_deep_free (list);
 
 		list = bacon_video_widget_get_subtitles (totem->bvw);
 		current = bacon_video_widget_get_subtitle (totem->bvw);
-		g_message ("current_sub: %d", current);
 		sub_menu = create_submenu (totem, list, current, FALSE);
 		totem_g_list_deep_free (list);
 	}
@@ -2722,15 +2824,15 @@ totem_callback_connect (Totem *totem)
 	/* Screenshot dialog */
 	item = glade_xml_get_widget (totem->xml, "dialog2");
 	g_signal_connect (G_OBJECT (item), "delete-event",
-			G_CALLBACK (hide_screenshot), (gpointer) totem);
+			G_CALLBACK (hide_screenshot), totem);
 	item = glade_xml_get_widget (totem->xml, "radiobutton1");
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_radiobutton_shot_toggled),
-			(gpointer) totem);
+			totem);
 	item = glade_xml_get_widget (totem->xml, "radiobutton2");
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_radiobutton_shot_toggled),
-			(gpointer) totem);
+			totem);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (item), TRUE);
 
 	/* Controls */
@@ -2841,22 +2943,22 @@ totem_callback_connect (Totem *totem)
 	/* Playlist Disappearance, woop woop */
 	g_signal_connect (G_OBJECT (totem->playlist),
 			"response", G_CALLBACK (toggle_playlist_from_playlist),
-			(gpointer) totem);
+			totem);
 	g_signal_connect (G_OBJECT (totem->playlist), "delete-event",
 			G_CALLBACK (toggle_playlist_from_playlist),
-			(gpointer) totem);
+			totem);
 
 	/* Playlist */
 	g_signal_connect (G_OBJECT (totem->playlist),
 			"changed", G_CALLBACK (playlist_changed_cb),
-			(gpointer) totem);
+			totem);
 	g_signal_connect (G_OBJECT (totem->playlist),
 			"current-removed", G_CALLBACK (current_removed_cb),
-			(gpointer) totem);
+			totem);
 	g_signal_connect (G_OBJECT (totem->playlist),
 			"repeat-toggled",
 			G_CALLBACK (playlist_repeat_toggle_cb),
-			(gpointer) totem);
+			totem);
 
 	/* DVD menu callbacks */
 	item = glade_xml_get_widget (totem->xml, "dvd_root_menu");
@@ -2940,6 +3042,9 @@ video_widget_create (Totem *totem)
 		g_free (msg);
 	}
 
+	totem_preferences_tvout_setup (totem);
+	totem_preferences_visuals_setup (totem);
+
 	container = glade_xml_get_widget (totem->xml, "frame2");
 	gtk_container_add (GTK_CONTAINER (container),
 			GTK_WIDGET (totem->bvw));
@@ -2960,10 +3065,22 @@ video_widget_create (Totem *totem)
 			"title-change",
 			G_CALLBACK (on_title_change_event),
 			totem);
+	g_signal_connect (G_OBJECT(totem->bvw),
+			"channels-change",
+			G_CALLBACK (on_channels_change_event),
+			totem);
 	g_signal_connect (G_OBJECT (totem->bvw),
 			"tick",
 			G_CALLBACK (update_current_time),
 			totem);
+	g_signal_connect (G_OBJECT (totem->bvw),
+			"got-metadata",
+			G_CALLBACK (on_got_metadata_event),
+			totem);
+	g_signal_connect (G_OBJECT (totem->bvw),
+			"buffering",
+			G_CALLBACK (on_buffering_event),
+			totem);
 
 	/* Events for the widget video window as well */
 	gtk_widget_add_events (GTK_WIDGET (totem->bvw), GDK_KEY_PRESS_MASK);
@@ -2988,7 +3105,7 @@ video_widget_create (Totem *totem)
 
 	gtk_widget_show (GTK_WIDGET (totem->bvw));
 
-	totem_preferences_tvout_setup (totem);
+	gtk_widget_set_size_request (container, 0, 0);
 }
 
 GtkWidget *
@@ -3088,7 +3205,6 @@ static void
 process_command_line (BaconMessageConnection *conn, int argc, char **argv)
 {
 	int i, command;
-	gboolean replace_done = FALSE;
 	char *line, *full_path;
 
 	if (argc == 1)
@@ -3154,12 +3270,15 @@ main (int argc, char **argv)
 	GConfClient *gc;
 	GError *err = NULL;
 	GdkPixbuf *pix;
-	int retval;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	/* Please also translate your language into its native language
+	 * in src/languages.h
+	 * for example: French -> Francais
+	 * use C UTF-8 strings */
 	g_set_application_name (_("Totem Movie Player"));
 
 	/* FIXME See http://bugzilla.gnome.org/show_bug.cgi?id=111349 */
@@ -3191,7 +3310,7 @@ main (int argc, char **argv)
 		str = g_strdup_printf (_("Totem couln't initialise the \n"
 					"configuration engine:\n%s"),
 				err->message);
-		totem_action_error_and_exit (str, totem);
+		totem_action_error_and_exit (str, NULL);
 		g_error_free (err);
 		g_free (str);
 	}
@@ -3303,6 +3422,7 @@ main (int argc, char **argv)
 
 	/* Show ! (again) the video widget this time. */
 	video_widget_create (totem);
+	long_action ();
 
 	/* The prefs after the video widget is connected */
 	totem_setup_preferences (totem);
@@ -3314,8 +3434,7 @@ main (int argc, char **argv)
 		else
 			totem_action_set_mrl (totem, NULL);
 	} else {
-		if (totem_action_restore_pl (totem) == FALSE)
-			totem_action_set_mrl (totem, NULL);
+		totem_action_restore_pl (totem);
 	}
 
 #ifdef HAVE_REMOTE
